package org.w3c.rdf.util;

import org.w3c.rdf.model.*;
import org.w3c.tools.crypt.*;
import java.security.*;
import java.util.Enumeration;

/**
 * An implementation of the algorithms for computing digests of triples and models.
 */

public class RDFDigestUtil {

  /**
   * @returns the digest of an RDF node
   */
  public static Digest getNodeDigest(RDFNode n) {
    
    return new Md5Digest(n.getLabel());
  }

  private static Digest reuseOrGetDigest(RDFNode n) {

    return n instanceof Digestable ? ((Digestable)n).getDigest() : getNodeDigest(n);
  }

  /**
   * @returns the standard MD5 digest of an RDF triple
   */
  public static Digest getTripleDigest(Triple t) {

    byte[] digest = reuseOrGetDigest(t.subject()).getDigestBytes();
    byte[] digestPred = reuseOrGetDigest(t.predicate()).getDigestBytes();
    byte[] digestObj = reuseOrGetDigest(t.object()).getDigestBytes();
	
    DigestUtil.xor(digest, digestPred, 1);
    DigestUtil.xor(digest, digestObj, (t.object() instanceof Literal ? 2 : 3));

    try {
      return new Md5Digest(digest);
    } catch (DigestException de) {
      throw new DigestAlgorithmMismatch();
    }
  }

  /**
   * @returns the standard digest of an model assuming that all digests are in MD5
   */
  public static Digest getModelDigest(RDFModel m) {

    byte[] digest = null;
    Enumeration en = m.elements();

    if(!en.hasMoreElements())
      return null;

    while(en.hasMoreElements()) {

      Triple t = (Triple)en.nextElement();
      Digest d = t instanceof Digestable ? ((Digestable)t).getDigest() : getTripleDigest(t);
      if(digest == null)
	digest = d.getDigestBytes();
      else
	DigestUtil.xor(digest, d.getDigestBytes());

    }

    try {
      return new Md5Digest(digest);
    } catch (DigestException de) {
      throw new DigestAlgorithmMismatch();
    }
  }

}
